package dev.fitko.fitconnect.client.util;

import dev.fitko.fitconnect.api.exceptions.client.FitConnectInitialisationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.yaml.snakeyaml.Yaml;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.InvalidPathException;
import java.nio.file.Path;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class RessourceLoadingUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(RessourceLoadingUtils.class);

    public static <T> T loadYaml(String path, Class<T> returnType) {
        try {
            final InputStream resource = RessourceLoadingUtils.class.getClassLoader().getResourceAsStream(path);
            return new Yaml().loadAs(resource, returnType);
        } catch (final Exception e) {
            throw new FitConnectInitialisationException("Yaml resource from path  " + path + " could not be loaded", e);
        }
    }

    public static List<Path> getDataSchemaPathsFromDir(final String submissionDataSchemaPath) {
        if (submissionDataSchemaPath == null) {
            return Collections.emptyList();
        }
        try (final Stream<Path> paths = Files.walk(Path.of(submissionDataSchemaPath))) {
            return paths.filter(Files::isRegularFile).collect(Collectors.toList());
        } catch (final IOException | InvalidPathException e) {
            throw new FitConnectInitialisationException("Could not read submission data schemas from path " + submissionDataSchemaPath, e);
        }
    }

    public static String readSchemaResourceFromPath(String schemaPath) {
        return new String(readResourceToByteArray(schemaPath));
    }

    public static List<String> readSchemaResourcesFromPaths(List<String> schemaPaths) {
        return schemaPaths.stream()
                .map(RessourceLoadingUtils::readResourceToByteArray)
                .map(String::new)
                .collect(Collectors.toList());
    }

    public static String readKeyFromPath(final String keyPath) {
        try {
            return Files.readString(Path.of(keyPath));
        } catch (final IOException | NullPointerException e) {
            LOGGER.error("Could not read key from path '{}'. Please provide a valid path in your config.yml", keyPath);
            throw new FitConnectInitialisationException(e.getMessage(), e);
        }
    }

    public static byte[] readResourceToByteArray(final String resourcePath) {
        try (final InputStream in = RessourceLoadingUtils.class.getResourceAsStream(resourcePath)) {
            return Objects.requireNonNull(in).readAllBytes();
        } catch (final IOException | NullPointerException exception) {
            throw new FitConnectInitialisationException("Failed loading resource " + resourcePath, exception);
        }
    }

}
